#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

#fonction affine 3e
import random
from math import *
from ..outils.Affichage import decimaux
from ..outils.Fractions import Fractions  #fractions pyromaths

def extreme(a,b,xmin,xmax,ymin,ymax):
#donne les extremités de la droite passant par a et b (coordonnées)
    res=[]
    x1=float(a[0])
    x2=float(b[0])
    y1=float(a[1])
    y2=float(b[1])
    coef=float((y1-y2)/(x1-x2))
    if coef != 0:
        xsort1=float(x1+(ymin-y1)/coef) #abscisse du point d'ordonnée ymin
        if xsort1 >=xmin and xsort1<=xmax and not(xsort1,ymin) in res:
            res.append((xsort1,ymin))
        xsort2=float(x2+(ymax-y2)/coef) #abscisse du point d'ordonnée ymax
        if xsort2>=xmin and xsort2<=xmax and not(xsort2,ymax) in res:
            res.append((xsort2,ymax))
        ysort1=float(y1+coef*(xmin-x1))  #ordonnée du point d'abscisse xmin
        if ysort1 >=ymin and ysort1<=ymax and not (xmin,ysort1)in res:
            res.append((xmin,ysort1))
        ysort2=float(y2+coef*(xmax-x2))  #ordonnée du point d'abscisse xmax
        if ysort2 >=ymin and ysort2<=ymax and not(xmax,ysort2) in res:
            res.append((xmax,ysort2))
    else:
        res=[(xmin,y1),(xmax,y1)]
    return res

def vecdir(A,B):
    #retourne sous forme de liste le vecteur directeur normé de la droite (AB)
    norm= sqrt((B[0]-A[0])**2+(B[1]-A[1])**2)
    u=[(B[0]-A[0])/norm,(B[1]-A[1])/norm]
    if u[0]<0:
        u[0]=-u[0]
        u[1]=-u[1]
    return u

def validedroite(A,B):
    #valide le choix du couple A B pour qu'ils ne soient pas "collés",
    # la droite (AB) ne sera ni horizontale ni verticale

    rep=True
    if abs(A[0]-B[0])<=1 and abs(A[1]-B[1])<=1:
        rep=False
    if A[0]==B[0] or A[1]==B[1]:
        rep=False
    if abs(A[0]-B[0])<1 or abs(A[1]-B[1])<1:
        rep=False
    return rep


def validec(A,B):
    #valide le choix du couple A B pour qu'ils ne soient pas "collés"
    rep=True
    if abs(A[0]-B[0])<=1 and abs(A[1]-B[1])<=1:
        rep=False
    return rep

def doublefleche(A,B):
    #trace une flèche "double" de justification en pointillés
    mid=(float((A[0]+B[0]))/2,float((A[1]+B[1]))/2)
    res1="\\psline[linestyle=dashed,linewidth=1.1pt]{->}"+str(A)+str(mid)+'\n '
    res2="\\psline[linestyle=dashed,linewidth=1.1pt]{->}"+str(mid)+str(B)
    res=res1+res2
    if A==B:
        res=""

    return res

def couple () :
    A=(float(random.randrange(-8,9))/2,float(random.randrange(-8,9))/2)
    B=(float(random.randrange(-8,9))/2,float(random.randrange(-8,9))/2)
    while not validec(A,B):
        B=(float(random.randrange(-8,9))/2,float(random.randrange(-8,9))/2)
    return (A,B)

def coupletrace () :
    A=(0,float(random.randrange(-4,5)))
    B=(float(random.randrange(-4,5)),float(random.randrange(-4,5)))
    while not validec(A,B):
        B=(float(random.randrange(-4,5)),float(random.randrange(-4,5)))
    return (A,B)

def couples ():
    #génère 6 points. Chaque couple correspondra a une droite ( (AB)(CD)(EF)).
    A=(float(random.randrange(-8,9))/2,float(random.randrange(-8,9))/2)
    B=(float(random.randrange(-8,9))/2,float(random.randrange(-8,9))/2)
    while not validedroite(A,B):
        B=(float(random.randrange(-8,9))/2,float(random.randrange(-8,9))/2)
    C=(0,float(random.randrange(-4,5)))
    while not (validec(A,C) and validec(B,C)):
        C=(0,float(random.randrange(-4,5)))
    D=(float(random.randrange(-4,5)),float(random.randrange(-4,5)))
    while not (validec(A,D) and validec(B,D) and validedroite(C,D)) :
        D=(float(random.randrange(-4,5)),float(random.randrange(-4,5)))
    E=(0,float(random.randrange(-8,9))/2)
    while not (validec(A,E) and validec(B,E) and validec(C,E) and validec(D,E)):
        E=(0,float(random.randrange(-8,9))/2)
    F=(float(random.randrange(-4,5)),float(random.randrange(-4,5)))
    while not (validec(A,F) and validec(B,F) and validec(C,F) and validec(D,F)and validedroite(E,F)):
        F=(float(random.randrange(-4,5)),float(random.randrange(-4,5)))
    return (A,B,C,D,E,F)

def tracedroite(A,B,xmin,xmax,ymin,ymax):
    #trace la droite (AB)
    l=extreme(A,B,xmin,xmax,ymin,ymax)
    return "\\psline "+str(l[0])+str(l[1])

def dansrep(A,xmin,xmax,ymin,ymax):
#Si le points est dans le repère
    res= False
    if A[0]>xmin and A[0]<xmax and A[1]>ymin and A[1]<ymax:
        res=True
    return res

def nomdroite(i,coordo):
    #place le nom de la droite (d_i) sur le graphique aux coordonnées coordo
    x0=coordo[0]
    y0=coordo[1]
    if x0!=0:
        x=x0/abs(x0)*(abs(x0)+0.5)
    else:
        x=0
    if y0!=0:
        y=y0/abs(y0)*(abs(y0)+0.5)
    else:
        y=0
    return "\\rput"+str((x,y))+"{($d_"+str(i)+"$)}"

def nom3droites(A,B,C,D,E,F,xmin,xmax,ymin,ymax):
    #place le nom des droites (AB), (CD), (EF) dans le graphique
    #les 3 instructions latex sont contenues dans une liste
    #évite la juxtaposition des écritures
    res=[]
    l1=extreme(A,B,xmin,xmax,ymin,ymax)
    res.append(nomdroite(1,l1[0]))
    l2=extreme(C,D,xmin,xmax,ymin,ymax)
    if validec(l1[0],l2[0]):
        memo=l2[0]      #se rappeler qu'on a utilisé l2[0]
        res.append(nomdroite(2,l2[0]))
    else:
        memo=l2[1]
        res.append(nomdroite(2,l2[1]))
    l3=extreme(E,F,xmin,xmax,ymin,ymax)
    if validec(l1[0],l3[0]) and validec(memo,l3[0]):
        res.append(nomdroite(3,l3[0]))
    else :
        res.append(nomdroite(3,l3[1]))
    return res

def isint(x):
    #est entier?
    res= False
    if int(x)==x:
        res=True
    return res

def isdemi(x):
    #est-ce une moitié d'entier
    res=False
    x=float(x)*2
    if isint(x):
        res= True

    return res

def coefdir(A,B):
    #donne le coefficient directeur x/y sous forme de liste [x,y]
    #Si y=1 on ecrira x sinon on écrira la fraction x/y
    x=float(B[1]-A[1])
    y=float(B[0]-A[0])

    if isint(x/y) or isdemi(x/y):
        res=[x/y,1]
    else:
        res=[x,y]
    if res[0]<0 and res[1]<0:
        res=[abs(res[0]),abs(res[1])]
    if res[1]<0:
        res=[-res[0],abs(res[1])]
    fr=Fractions.simplifie(Fractions(res[0],res[1]))

    return [fr.n,fr.d]

def anteimage(fonc,A,B):
    #Génère la 1ère question et sa réponse

    l=[' l\'image de ',' un nombre qui a pour image ',u' un antécédent de ']
    lcor=[' est l\'image de ',' a pour image ',u' est un antécédent de '] #liste pour le corrigé
    i=random.randrange(0,2)
    j=i
    if i==1:
        j=i+random.randrange(0,2)
    res=[]
    res.append('Donner '+l[j]+'$'+decimaux(str(A[i]))+'$'+' par la fonction '+'\\textit{'+fonc+'}.')
    res.append('$'+decimaux(str(A[abs(i-1)]))+'$' + lcor[j]+'$'+decimaux(str(A[i]))+'$'+' par la \\hbox{fonction '+'\\textit{'+fonc+'}}.')
    if i==0:
        res.append(doublefleche((A[0],0),A))
        res.append(doublefleche(A,(0,A[1])))

    else:
        res.append(doublefleche((0,A[1]),A))
        res.append(doublefleche(A,(A[0],0)))
    i=abs(i-1)
    j=i
    if i==1:
        j=i+random.randrange(0,2)
    res.append('Donner '+l[j]+'$'+decimaux(str(B[i]))+'$'+' par la fonction '+'\\textit{'+fonc+'}.')
    res.append('$'+decimaux(str(B[abs(i-1)]))+'$'+lcor[j]+'$'+decimaux(str(B[i]))+'$'+' par la \\hbox{fonction '+'\\textit{'+fonc+'}}.')
    if i==0:
        res.append(doublefleche((B[0],0),B))
        res.append(doublefleche(B,(0,B[1])))
    else:
        res.append(doublefleche((0,B[1]),B))
        res.append(doublefleche(B,(B[0],0)))
    return res

def tracefonc(f,i,A,B,xmin,xmax,ymin,ymax):
#A est sur l'axe des ordonnées, f est le nom de la fonction
#Génère la 2e queston et sa réponse
    u=coefdir(A,B)
    if A[1]>=0:
        b='+'+decimaux(str(A[1]))
    else:
        b=decimaux(str(A[1]))
    if u[1]==1:
        coef=decimaux(str(u[0]))
        if u[0]==-1:
            coef='-'
        if u[0]==1:
            coef=''
        x1=decimaux(str(B[0]))
        y1=decimaux(str(B[1]))
        u[0]=u[0]*B[0]
    else:
        B=(u[1],u[0]+float(A[1]))
        if not dansrep(B,xmin,xmax,ymin,ymax):
            B=(-u[1],-u[0]+float(A[1]))

        x1=decimaux(str(B[0]))
        y1=decimaux(str(B[1]))
        if u[0]>0:
            coef='\\dfrac{'+decimaux(str(u[0]))+'}{'+decimaux(str(u[1]))+'}'
        else:
            coef='-\\dfrac{'+decimaux(str(abs(u[0])))+'}{'+decimaux(str(u[1]))+'}'

    x0='0'
    y0=b
    if coef=='' or (coef=='-' and B[0]>0) :
        st='On sait que $'+f+'(0)='+decimaux(str(A[1]))+'$ et $'+f+'('+x1+')='+coef+x1+b+'='+y1+'$.'
    elif coef=='-' and B[0]<0:
        st='On sait que $'+f+'(0)='+decimaux(str(A[1]))+'$ et $'+f+'('+x1+')='+coef+'('+x1+')'+b+'='+y1+'$.'
    elif B[0]<0:
        st='On sait que $'+f+'(0)='+decimaux(str(A[1]))+'$ et $'+f+'('+x1+')='+coef+' \\times ('+x1+')'+b+'='+decimaux(str(u[0]))+b+'='+y1+'$.'
    else:
        st='On sait que $'+f+'(0)='+decimaux(str(A[1]))+'$ et $'+f+'('+x1+')='+coef+' \\times '+x1+b+'='+decimaux(str(u[0]))+b+'='+y1+'$.'

    l=[u'Tracer la droite représentative ($d_'+str(i)+'$) de la fonction $'+f+':x\\longmapsto '+coef+'x'+b+'$.',
       st,
       '\\psdot [dotsize=4.5pt,dotstyle=x]'+str(A),
       '\\psdot [dotsize=4.5pt,dotstyle=x]'+str(B),
       ]

    return l

def exprfonc(f,i,A,B):
#Génère la 3e question.
#A est sur l'axe des ordonnées, f est le nom de la fonction
    u=coefdir(A,B)
    if A[1]>=0:
        b='+'+decimaux(str(A[1]))
    else:
        b=decimaux(str(A[1]))
    if u[1]==1:
        coef=decimaux(str(u[0]))
        if u[0]==-1:
            coef='-' #utilisé dans l'expression de la fonction
        if u[0]==1:
            coef=''
        coefres=decimaux(str(u[0]))  #résultat utilisé pour a
    else:
        if u[0]>0:
            coef='\\dfrac{'+decimaux(str(u[0]))+'}{'+decimaux(str(u[1]))+'}'
        else:
            coef='-\\dfrac{'+decimaux(str(abs(u[0])))+'}{'+decimaux(str(u[1]))+'}'
        coefres=coef

    if A[1]-B[1]>0:
        deltay='+'+decimaux(str(A[1]-B[1]))
    else :
        deltay=decimaux(str(A[1]-B[1]))
    if A[0]-B[0]>0:
        deltax='+'+decimaux(str(A[0]-B[0]))
    else:
        deltax=decimaux(str(A[0]-B[0]))

    if float(B[0])<0 :
       mid11=float(B[0])-0.75
       mid12=float((B[1]+A[1]))/2   #milieu de la flèche verticale
    else:
        mid11=float(B[0])+0.75
        mid12=float((B[1]+A[1]))/2
    if float(B[0])*float(u[1]/u[0])>0 :
        mid21=float((A[0]+B[0]))/2
        mid22=A[1]-0.6               #milieu de la flèche horizontale
    else :
        mid21=float((A[0]+B[0]))/2
        mid22=A[1]+0.5
    if mid12 < 0 and mid12 >-0.8:
        mid12=-1
    if mid12 >= 0 and mid12 < 0.5:
        mid12=0.5
    if mid11 < 0 and mid11 >-0.8:
        mid11=-1
    if mid11 >=0 and mid11 < 0.5:
        mid11=0.5
    if mid21 < 0 and mid21 >-0.8:
        mid21=-1
    if mid21 >=0 and mid21 < 0.5:
        mid21=0.5
    if mid22 < 0 and mid22 >-0.8:
        mid22=-1
    if mid22 >= 0 and mid22 < 0.5:
        mid22=0.5
    mid1=(mid11,mid12)
    mid2=(mid21,mid22)

    l=[u'Déterminer l\'expression de la fonction $'+f+u'$ représentée ci-contre par la droite ($d_'+str(i)+'$).',
       u'On lit l\'ordonnée à l\'origine et le coefficient de la fonction affine sur le graphique.\\\ ',
       '$'+f+'(x)=ax+b$ ' +'avec $b='+ decimaux(str(A[1]))+'$ et $a='+'\\dfrac{'+deltay+'}{'+deltax+'}='+coefres+'$.\\\ ',
       'L\'expression de la fonction $'+f+'$ est $'+f+'(x)='+coef+'x'+b+'$.',
       doublefleche(B,(B[0],A[1])),
       doublefleche((B[0],A[1]),A),
       '\\rput'+str(mid1)+'{('+deltay+')}',
       '\\rput'+str(mid2)+'{('+deltax+')}']

    return l


def affine():
    #Génère l'exercice
    xmin,xmax,ymin,ymax=-5,5,-5,5
    f=['f','g','h','k','l','u']
    rgfonc1=random.randrange(0,6)
    fonc1=f[rgfonc1]
    (A,B,C,D,E,F)=couples()
    l=anteimage(fonc1,A,B) #lecture d'image d'antécédent
    fonc2=f[(rgfonc1+1)%6]
    l2=tracefonc(fonc2,2,C,D,xmin,xmax,ymin,ymax) #représenter une fonction
    fonc3=f[(rgfonc1+2)%6]
    l3=exprfonc(fonc3,3,E,F)
    noms=nom3droites(A,B,C,D,E,F,xmin,xmax,ymin,ymax)
    exo=["\\exercice", "\\parbox{0.5\\linewidth}{",
         u"($d_1$) est la droite représentative de la fonction $"+fonc1+"$.",
         "\\begin{enumerate}",
         "\\item "+l[0],
         "\\item "+l[4],
         "\\item "+l2[0],
         "\\item "+l3[0],
         "\\end{enumerate}}\\hfill",
         "\\parbox{0.45\\linewidth}{",
         "\\psset{unit=0.8cm}",
         "\\begin{pspicture}"+str((xmin,ymin))+str((xmax,ymax)),
         "\\psgrid[subgriddiv=2, gridlabels=8pt](0,0)"+str((xmin,ymin))+str((xmax,ymax)),
         "\\psline[linewidth=1.2pt]{->}"+str((xmin,0))+str((xmax,0)),
         "\\psline[linewidth=1.2pt]{->}"+str((0,ymin))+str((0,ymax)),
         tracedroite(A,B,xmin,xmax,ymin,ymax),
         noms[0],
         tracedroite(E,F,xmin,xmax,ymin,ymax),
         noms[2],
         "\\end{pspicture}}"]

    cor=["\\exercice*", "\\setlength{\\columnsep}{2mm}",
         "\\begin{multicols}{2}\\noindent \\small",
         u"($d_1$) est la droite représentative de la fonction $"+fonc1+"$.",
         "\\begin{enumerate}",
         "\\item "+l[1],
         "\\item "+l[5],
         "\\item\n\\begin{flushleft}\n"+l2[1]+"\n\\end{flushleft}",
         "\\item\n\\begin{flushleft}\n"+l3[1],
         l3[2],
         l3[3]+"\n\\end{flushleft}",
         "\\end{enumerate}",
         "\\end{multicols}",
         "\\vspace{0.45cm}",
         "\\begin{minipage}{0.5\\linewidth}",
         "\\psset{unit=0.7cm}",
         "\\begin{center}",
         "\\begin{pspicture}"+str((xmin,ymin))+str((xmax,ymax)),
         "\\psgrid[subgriddiv=2, gridlabels=8pt](0,0)"+str((xmin,ymin))+str((xmax,ymax)),
         "\\psline[linewidth=1.2pt]{->}"+str((xmin,0))+str((xmax,0)),
         "\\psline[linewidth=1.2pt]{->}"+str((0,ymin))+str((0,ymax)),
         tracedroite(A,B,xmin,xmax,ymin,ymax),
         noms[0],
         tracedroite(C,D,xmin,xmax,ymin,ymax),
         noms[1],
         l[2],
         l[3],
         l[6],
         l[7],
         l2[2],
         l2[3],
         "\\end{pspicture}",
         "\\end{center}",
         "\\end{minipage}",
         "\\begin{minipage}{0.5\\linewidth}",
         "\\psset{unit=0.7cm}",
         "\\begin{center}",
         "\\begin{pspicture}"+str((xmin,ymin))+str((xmax,ymax)),
         "\\psgrid[subgriddiv=2, gridlabels=8pt](0,0)"+str((xmin,ymin))+str((xmax,ymax)),
         "\\psline[linewidth=1.2pt]{->}"+str((xmin,0))+str((xmax,0)),
         "\\psline[linewidth=1.2pt]{->}"+str((0,ymin))+str((0,ymax)),
         tracedroite(E,F,xmin,xmax,ymin,ymax),
         noms[2],
         l3[4],
         l3[5],
         l3[6],
         l3[7],
         "\\end{pspicture}",
         "\\end{center}",
         "\\end{minipage}",
         "\\vspace{0.45cm}"]
    return exo,cor

def tex_affine():
    return affine()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

from ..outils import Arithmetique
from ..outils.Priorites import OperateurPrioritaire
from ..outils.TeXMiseEnForme import Affichage
import random
from ..classes.Fractions import Fractions #Classe Fractions de pyromaths


def sommes_fractions_4e(op, level):
    '''Choisit des valeurs aléatoires pour effectuer une somme ou une différence
    de fractions en fonction du niveau de difficulté souhaité (de 1 à 4) et renvoie
    l'énoncé et le corrigé au format TeX

    @param op: '+' ou '-'
    @param level: niveau de difficulté :
                  1- Fractions positives et dénominateur de l'une multiple de l'autre
                  2- Fractions positives et dénominateurs non multiples
                  3- Fractions avec des nombres relatifs
                  4- Fractions avec des nombres relatifs et résultats simplifiable
    '''

    while True:
        (n1, d1, n2, d2) = (2, 2, 2, 2)
#        import pdb; pdb.set_trace()
        while True:
            if level == 1:
                n1 = random.randrange(1, 16)
                d1 = random.randrange(1, 9)
                n2 = random.randrange(1, 16)
                d2 = d1*random.randrange(2, 11)
                if random.randrange(2):
                    d1, d2 = d2, d1
            elif level == 2:
                n1 = Arithmetique.valeur_alea(1, 16)
                d1 = Arithmetique.valeur_alea(1, 40)
                n2 = Arithmetique.valeur_alea(1, 16)
                d2 = Arithmetique.valeur_alea(1, 40)
            else:
                while True:
                    neg=[(-1,1)[random.randrange(2)] for x in range(4)]
                    if neg[0]<0 or neg[1]<0 or neg[2]<0 or neg[3]<0:
                        break
                n1 = random.randrange(1, 16)*neg[0]
                d1 = random.randrange(1, 40)*neg[1]
                n2 = random.randrange(1, 16)*neg[2]
                d2 = random.randrange(1, 40)*neg[3]
            fr1 = Fractions(n1, d1)
            fr2 = Fractions(n2, d2)
            if Arithmetique.pgcd(fr1.n, fr1.d) == 1 and \
                Arithmetique.pgcd(fr2.n, fr2.d) == 1 and \
                (level == 1 or (Arithmetique.pgcd(fr1.d, fr2.d) != abs(fr1.d) \
                                and Arithmetique.pgcd(fr1.d, fr2.d) != abs(fr2.d))):
                if op == "+":
                    simplifiable = abs(fr1.d * fr2.d) != \
                                        abs(Fractions.simplifie(fr1 + fr2).d)
                else:
                    simplifiable = abs(fr1.d * fr2.d) != \
                                        abs(Fractions.simplifie(fr1 - fr2).d)
                if level == 1 or (simplifiable and level == 4) or \
                                            (not simplifiable and level < 4):
                    break

        l = [fr1, op, fr2]
        (cor, res, niveau) = OperateurPrioritaire(l, 4, solution=[])
        if niveau >= 4:
            break
    fr1 = Fractions(n1, d1)
    fr2 = Fractions(n2, d2)
    return ([fr1, op, fr2], cor, res)

def produits_fractions_4e(op, level):
    '''Choisit des valeurs aléatoires pour effectuer un produit ou un quotient de
    fractions en fonction du niveau de difficulté souhaité (de 1 à 4) et renvoie
    l'énoncé et le corrigé au format TeX

    @param op: '*' ou '/'
    @param level: niveau de difficulté :
                  1- Fractions positives non décomposables
                  2- Fractions avec des nombres relatifs non décomposables
                  3- Fractions positives à décomposer
                  4- Fractions avec des nombres relatifs à décomposer
    '''

    while True:
        (n1, d1, n2, d2) = (2, 2, 2, 2)
        while True:
            n1=d1=n2=d2=a=b=2
            if level == 3 or level == 4:
                while Arithmetique.pgcd(a,b)>1:
                    a=random.randrange(2,11)
                    b=random.randrange(2,11)
            else:
                a, b = 1, 1
            if op == "*":
                if level == 1 or level == 3:
                    while Arithmetique.pgcd(n1*a,d1*b)>1:
                        n1=random.randrange(1,11)
                        d1=random.randrange(2,11)
                    while Arithmetique.pgcd(n2*b,d2*a)>1:
                        n2=random.randrange(1,11)
                        d2=random.randrange(2,11)
                elif level == 2 or level == 4:
                    while True:
                        neg=[(-1,-1,1)[random.randrange(3)] for x in range(4)]
                        if neg[0]<0 or neg[1]<0 or neg[2]<0 or neg[3]<0:
                            break
                    while Arithmetique.pgcd(n1*a,d1*b)>1:
                        n1=random.randrange(1,11)*neg[0]
                        d1=random.randrange(2,11)*neg[1]
                    while Arithmetique.pgcd(n2*b,d2*a)>1:
                        n2=random.randrange(1,11)*neg[2]
                        d2=random.randrange(2,11)*neg[3]
                fr1 = Fractions(n1*a, d1*b)
                fr2 = Fractions(n2*b, d2*a)
                simplifiable = abs(fr1.d * fr2.d) != Fractions.simplifie(fr1 *
                                                                         fr2).d
            else:
                if level == 1 or level == 3:
                    while Arithmetique.pgcd(n1*a,d1*b)>1:
                        n1=random.randrange(1,11)
                        d1=random.randrange(2,11)
                    while Arithmetique.pgcd(n2*a,d2*b)>1:
                        n2=random.randrange(1,11)
                        d2=random.randrange(2,11)
                else:
                    while Arithmetique.pgcd(n1*a,d1*b)>1:
                        n1=random.randrange(-11,11)
                        d1=random.randrange(2,11)*(-1,1)[random.randrange(2)]
                    while Arithmetique.pgcd(n2*a,d2*b)>1:
                        n2=random.randrange(1,11)*(-1,1)[random.randrange(2)]
                        d2=random.randrange(2,11)*(-1,1)[random.randrange(2)]
                fr1 = Fractions(n1*a, d1*b)
                fr2 = Fractions(n2*a, d2*b)
                simplifiable = abs(fr1.d * fr2.n) != \
                                            Fractions.simplifie(fr1 / fr2).d
            if (simplifiable and level>2) or (not simplifiable and level<=2):
                break
        l = [fr1, op, fr2]

        (cor, res, niveau) = OperateurPrioritaire(l, 4, solution=[])
        if niveau >= 4:
            break
    return (l, cor, res)


def valeurs_priorites_fractions(nb, entier=1):  # renvoie les 2 listes contenant les opérateurs et les opérandes.
    listoperateurs = [
        "+",
        "*",
        "-",
        "/",
        '(',
        '(',
        '(',
        '(',
        ')',
        ')',
        ')',
        ')',
        ]
    loperateurs = []
    loperandes = []
    i = 0  #nombre d'opérateurs créés
    p = 0  #nombre de parenthèses ouvertes
    cpt = 0  #compteur pour éviter que le programme ne boucle.
    while i < nb - 1:
        cpt = cpt + 1
        if cpt > 10:  #On recommence
            (cpt, i, p, loperateurs) = (0, 0, 0, [])
        if p:
            if loperateurs[-1] == '(':  # On n'écrit pas 2 parenthèses à suivre
                operateur = listoperateurs[random.randrange(4)]
            else:
                operateur = listoperateurs[random.randrange(12)]
        elif loperateurs == []:

            # On ne commence pas par une parenthèse

            operateur = listoperateurs[random.randrange(4)]
        else:
            operateur = listoperateurs[random.randrange(8)]
        if nb > 3:
            test = ('-*/').find(operateur) >= 0 and loperateurs.count(operateur) < \
                1 or operateur == "+" and loperateurs.count(operateur) < \
                2
        else:
            test = ('-*/+').find(operateur) >= 0 and loperateurs.count(operateur) < \
                1
        if test:

            #On n'accepte pas plus de 1 produit, différence, quotient et de 2 sommes ou parenthèses par calcul.

            if i == 0 or loperateurs[-1] != '(' or ('*/').find(operateur) < \
                0:  #pas de * ou / dans une parenthèse.
                i = i + 1
                loperateurs.append(operateur)
        elif operateur == '(' and (')+').find(loperateurs[-1]) < 0:

            #Il ne peut y avoir de ( après une ) ou après un +

            p = p + 1
            loperateurs.append(operateur)
        elif operateur == ')':
            p = p - 1
            loperateurs.append(operateur)
    while p > 0:
        loperateurs.append(')')
        p = p - 1
    loperandes = []
    for i in range(nb):
        (n, d) = (2, 2)
        while Arithmetique.pgcd(n, d) != 1 or abs(d) == 1:
            n = Arithmetique.valeur_alea(-16, 16)
            d = -Arithmetique.valeur_alea(-40, 40)
        loperandes.append(Fractions(n, d))
    exercice = [loperandes[0]]
    i = 1
    j = 0
    while i < len(loperandes) or j < len(loperateurs):
        if j < len(loperateurs):
            exercice.append(loperateurs[j])
            j = j + 1
        while j < len(loperateurs) and (loperateurs[j] == '(' or
                loperateurs[j - 1] == ')'):
            exercice.append(loperateurs[j])
            j = j + 1
        if i < len(loperandes):
            exercice.append(loperandes[i])
            i = i + 1
    return exercice


def exo_sommes_fractions():
    exo = ["\\exercice",
           u"Effectuer les calculs suivants et donner le résultat sous la forme d'une fraction simplifiée :",
           "\\begin{multicols}{4}", "  \\noindent%"]
    cor = ["\\exercice*",
           u"Effectuer les calculs suivants et donner le résultat sous la forme d'une fraction simplifiée :",
           "\\begin{multicols}{4}", "  \\noindent%"]
    op = ["+", "-","+", "-","+", "-","+", "-"]
    for i in range(8):
        if i%2:
            (l, sol, res) = sommes_fractions_4e(op.pop(0), i//2+1)
        else:
            (l, sol, res) = sommes_fractions_4e(op.pop(random.randrange(2)),
                                                i//2+1)
        exo.append("\\[\\thenocalcul = %s\\]" % Affichage(l))
        cor.append("\\[\\thenocalcul = %s\\]" % Affichage(l))
        for l in sol:
            if l == sol[-1]:
                cor.append("\\[\\boxed{\\thenocalcul = %s}\\]" % l)
            else:
                cor.append("\\[\\thenocalcul = %s\\]" % l)
        exo.append("\\stepcounter{nocalcul}%")
        cor.append("\\stepcounter{nocalcul}%")
    exo.append("\\end{multicols}\n")
    cor.append("\\end{multicols}\n")
    return (exo, cor)


def exo_produits_fractions():
    exo = ["\\exercice",
           u"Effectuer les calculs suivants et donner le résultat sous la forme d'une fraction simplifiée :",
           "\\begin{multicols}{4}", "  \\noindent%"]
    cor = ["\\exercice*",
           u"Effectuer les calculs suivants et donner le résultat sous la forme d'une fraction simplifiée :",
           "\\begin{multicols}{4}", "  \\noindent%"]
    op = ["*", "/","*", "/","*", "/","*", "/"]
    for i in range(8):
        if i%2:
            (l, sol, res) = produits_fractions_4e(op.pop(0), i//2+1)
        else:
            (l, sol, res) = produits_fractions_4e(op.pop(random.randrange(2)), i//2+1)
        exo.append("\\[\\thenocalcul = %s\\]" % Affichage(l))
        cor.append("\\[\\thenocalcul = %s\\]" % Affichage(l))
        for l in sol:
            if l == sol[-1]:
                cor.append("\\[\\boxed{\\thenocalcul = %s}\\]" % l)
            else:
                cor.append("\\[\\thenocalcul = %s\\]" % l)
        exo.append("\\stepcounter{nocalcul}%")
        cor.append("\\stepcounter{nocalcul}%")
    exo.append("\\end{multicols}\n")
    cor.append("\\end{multicols}\n")
    return (exo, cor)


def exo_priorites_fractions():
    exo = ["\\exercice",
           u"Effectuer les calculs suivants et donner le résultat sous la forme d'une fraction simplifiée :",
           "\\begin{multicols}{3}", "  \\noindent%"]
    cor = ["\\exercice*",
           u"Effectuer les calculs suivants et donner le résultat sous la forme d'une fraction simplifiée :",
           "\\begin{multicols}{3}", "  \\noindent%"]
    for i in range(6):
        while True:
            l = valeurs_priorites_fractions(3)
            (sol, res, niveau) = OperateurPrioritaire(l, 4, solution=[])
            if niveau >= 4:
                break
        exo.append("\\[\\thenocalcul = %s\\]" % Affichage(l))
        cor.append("\\[\\thenocalcul = %s\\]" % Affichage(l))
        for l in sol:
            if l == sol[-1]:
                cor.append("\\[\\boxed{\\thenocalcul = %s}\\]" % l)
            else:
                cor.append("\\[\\thenocalcul = %s\\]" % l)
        exo.append("\\stepcounter{nocalcul}%")
        cor.append("\\stepcounter{nocalcul}%")
    exo.append("\\end{multicols}\n")
    cor.append("\\end{multicols}\n")
    return (exo, cor)

exo_sommes_fractions()



# -*- coding: utf-8 -*-

class Terme:
    def __init__(self, coeff, puiss = 1, var = 'x'):
        if (not (isinstance(puiss, int))) or (not (isinstance(var, str))):
            print "Erreur de définition ! La puissance doit être un nombre relatif !"
        self.puiss = puiss # Nombre entier
        self.coeff = coeff # Coeff devant le terme
        self.var = var # Lettre pour la var

    def __add__(self, other):
        if not (isinstance(other, Terme)):
            return str(self) + " + " + str(other)
        if (self.puiss == other.puiss) and (self.var == other.var):
            return Terme(self.coeff + other.coeff, self.puiss, self.var)
        else:
            return str(self) + " + " + str(other)

    def __radd__(self, other):
        return self + other

    def __sub__(self, other):
        if not (isinstance(other, Terme)):
            return str(self) + " + " + str(other)
        if (self.puiss == other.puiss) and (self.var == other.var):
            return Terme(self.coeff - other.coeff, self.puiss, self.var)
        else:
            return str(self) + " - " + str(other)

    def __rsub__(self, other):
        return -(self - other)

    def __mul__(self, other):
        if (isinstance(other, float)) or (isinstance(other, int)):
            return Terme(self.coeff * other, self.puiss, self.var)
        elif self.var == other.var:
            return Terme(self.coeff * other.coeff, self.puiss + other.puiss, self.var)
        else:
            return Terme(self.coeff * other.coeff, self.puiss + other.puiss, self.var + other.var)

    def __rmul__(self, other):
        return self * other

    def __div__(self, other):
        if (isinstance(other, float)) or (isinstance(other, int)):
            return Terme(self.coeff / other, self.puiss, self.var)
        elif self.var == other.var:
            return Terme(self.coeff / other.coeff, self.puiss - other.puiss, self.var)
        else:
            return str(self.coeff / other.coeff) + str(Terme(1, self.puiss, self.var)) + " / " + str(Terme(1, other.puiss, other.var))

    def __rdiv__(self, other):
        return self.inv() * other

    def inv(self):
        return Terme(1.0 / self.coeff, - self.puiss, self.var)

    def __str__(self):
        if self.coeff == 1:
            coeff = ''
        elif self.coeff == -1:
            coeff = '-'
        else:
            coeff = str(self.coeff) + ' '
        if self.puiss == 1:
            terme = self.var
        elif (self.puiss == 0) and ((self.coeff == 1) or (self.coeff == -1)):
            terme = '1'
        elif self.puiss == 0:
            terme = ''
        else:
            terme = self.var + u'^' + str(self.puiss)
        return coeff + terme
                



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

import math
import random
import string
from ..outils import Arithmetique, Affichage

#===============================================================================
# Poser des opérations
#===============================================================================


def valeurs():
    nba = Arithmetique.valeur_alea(111, 99999)
    while 1:
        nbb = Arithmetique.valeur_alea(111, 99999)
        if nbb - (nbb // 10) * 10:
            break
    puisb = Arithmetique.valeur_alea(-2, 0)
    nbb = nbb * 10 ** puisb
    deca = [str(nba)[i] for i in range(len(str(nba)))]
    decb = [str(nbb)[i] for i in range(len(str(nbb)))]
    if random.randrange(2):
        (nba, deca, nbb, decb) = (nbb, decb, nba, deca)
    if deca.count('.'):
        posa = deca.index('.')
    else:
        posa = len(deca)
    if decb.count('.'):
        posb = decb.index('.')
    else:
        posb = len(decb)

    lavtvirg = max(posa, posb)
    laprvirg = max(len(deca) - posa, len(decb) - posb)
    return (nba, nbb, deca, decb, lavtvirg, laprvirg)


def valeurs_prod():
    while 1:
        nba = Arithmetique.valeur_alea(101, 9999)
        if nba - (nba // 10) * 10:
            break
    puisa = Arithmetique.valeur_alea(-3, -1)
    while 1:
        nbb = Arithmetique.valeur_alea(101, 999)
        if nbb - (nbb // 10) * 10:
            break
    puisb = Arithmetique.valeur_alea(-3, -1)
    return (nba, nbb, puisa, puisb)


def lignes(ligne, deca, lavtvirg, laprvirg):
    if deca.count('.'):
        posa = deca.index('.')
    else:
        posa = len(deca)
    if posa < lavtvirg:
        for i in range(lavtvirg - posa):
            ligne.append('')
    for i in range(len(deca)):
        if deca[i] == '.':
            ligne.append(',')
        else:
            ligne.append(str(deca[i]))
    for i in range(laprvirg - (len(deca) - posa)):
        if ligne.count(','):
            ligne.append('0')
        else:
            ligne.append(',')
    return ligne


def mon_int(t):  # retourne un entier texte sous la forme d'un nombre, zéro sinon
    if t == '':
        t = 0
    elif ('1234567890').count(t):
        t = int(t)
    else:
        t = 0
    return t


def retenues_somme(ligne1, ligne2):
    lg = len(ligne1)
    ligne0 = ['' for i in range(lg)]
    for i in range(lg - 1):

        #on déplace la retenue pour qu'elle ne soit pas au-dessus de la virgule

        if ligne1[(lg - i) - 1] == ',' and ligne0[(lg - i) - 1] == '1':
            ligne0[(lg - i) - 2] = '1'
        elif mon_int(ligne1[(lg - i) - 1]) + mon_int(ligne2[(lg - i) - 1]) + \
            mon_int(ligne0[(lg - i) - 1]) > 9:
            ligne0[(lg - i) - 2] = '1'
    return ligne0


def retenues_diff(ligne1, ligne2):
    lg = len(ligne1)
    ret = 0
    for i in range(lg - 1):
        if not (ligne1[(lg - i) - 1] == ',' and ret):
            if mon_int(ligne1[(lg - i) - 1]) < mon_int(ligne2[(lg - i) -
                    1]) + ret:
                ligne1[(lg - i) - 1] = '$_1$%s' % ligne1[(lg - i) - 1]
                tmpret = 1
            else:
                tmpret = 0
            if ret:
                ligne2[(lg - i) - 1] = '%s$_1$' % ligne2[(lg - i) - 1]
            ret = tmpret
    return (ligne1, ligne2)


def tex_somme(exo, cor):
    (ligne1, ligne2) = ([''], ['+'])
    (nba, nbb, deca, decb, lavtvirg, laprvirg) = valeurs()
    total = nba + nbb
    dectotal = [str(total)[i] for i in range(len(str(total)))]
    if dectotal.index('.') <= lavtvirg:
        ligne3 = ['']
    else:
        ligne3 = []
    ligne1 = lignes(ligne1, deca, lavtvirg, laprvirg)
    ligne2 = lignes(ligne2, decb, lavtvirg, laprvirg)
    ligne3 = lignes(ligne3, dectotal, lavtvirg, laprvirg)
    ligne0 = retenues_somme(ligne1, ligne2)
    if ligne0[0] == '1':
        ligne0[0] = '\\tiny 1'
    exo.append('\\item La somme des termes %s et %s.\\par' % (Affichage.decimaux(nba),
             Affichage.decimaux(nbb)))
    cor.append('\\item La somme des termes %s et %s.\\par' % (Affichage.decimaux(nba),
             Affichage.decimaux(nbb)))
    cor.append('\\begin{tabular}[t]{*{%s}{c}}' % (lavtvirg +
             laprvirg + 1))
    cor.append('%s \\\\' % ' & \\tiny '.join(ligne0))
    cor.append('%s \\\\' % ' & '.join(ligne1))
    cor.append('%s \\\\\n\\hline' % ' & '.join(ligne2))
    cor.append('%s \\\\' % ' & '.join(ligne3))
    cor.append('\\end{tabular}\\par')
    formule = '%s+%s = %s' % (Affichage.decimaux(nba, 1),
                     Affichage.decimaux(nbb, 1), Affichage.decimaux(nba +
                     nbb, 1))
    cor.append((u'\\[ \\boxed{%s} \\] ').expandtabs(2 * 3) % (formule))

def tex_difference(exo, cor):
    (ligne1, ligne2) = ([''], ['-'])
    (nba, nbb, deca, decb, lavtvirg, laprvirg) = valeurs()
    if nba < nbb:
        (nba, nbb, deca, decb) = (nbb, nba, decb, deca)
    total = nba - nbb
    dectotal = [str(total)[i] for i in range(len(str(total)))]
    if dectotal.index('.') <= lavtvirg:
        ligne3 = ['']
    else:
        ligne3 = []
    ligne1 = lignes(ligne1, deca, lavtvirg, laprvirg)
    ligne2 = lignes(ligne2, decb, lavtvirg, laprvirg)
    ligne3 = lignes(ligne3, dectotal, lavtvirg, laprvirg)
    (ligne1, ligne2) = retenues_diff(ligne1, ligne2)
    exo.append(u"\\item La différence des termes %s et %s.\\par" %
             (Affichage.decimaux(nba), Affichage.decimaux(nbb)))
    cor.append(u"\\item La différence des termes %s et %s.\\par" %
             (Affichage.decimaux(nba), Affichage.decimaux(nbb)))
    cor.append('\\begin{tabular}[t]{*{%s}{c}}' % (lavtvirg +
             laprvirg + 1))
    cor.append('%s \\\\' % ' & '.join(ligne1))
    cor.append('%s \\\\\n\\hline' % ' & '.join(ligne2))
    cor.append('%s \\\\' % ' & '.join(ligne3))
    cor.append('\\end{tabular}\\par')
    formule = '%s-%s = %s' % (Affichage.decimaux(nba, 1),
                     Affichage.decimaux(nbb, 1), Affichage.decimaux(nba -
                     nbb, 1))
    cor.append((u'\\[ \\boxed{%s} \\] ').expandtabs(2 * 3) % (formule))

def pose_mult(nba, nbb):
    (ligne, total) = ([], 0)
    for i in range(int(math.log10(nbb)) + 1):
        sstotal = ((nbb - (nbb // 10) * 10) * nba) * 10 ** i
        total = total + sstotal
        ligne.append(sstotal)
        nbb = nbb // 10
    return (ligne, total)


def ligneprod(ligne, dec, lg):
    ligne.extend(['' for i in range((lg - len(dec)) - len(ligne))])
    ligne.extend(dec)
    return ligne


def tex_produit(exo, cor):
    (nba, nbb, puisa, puisb) = valeurs_prod()
    deca = [','.join(str(nba * 10 ** puisa).rsplit('.'))[i] \
                    for i in range(len(str(nba * 10 ** puisa)))]
    decb = [','.join(str(nbb * 10 ** puisb).rsplit('.'))[i] \
                    for i in range(len(str(nbb * 10 ** puisb)))]

    (dec3, total) = pose_mult(nba, nbb)
    (dec3bis, total) = pose_mult(nbb, nba)
    total = ((nba * 10 ** puisa) * nbb) * 10 ** puisb
    dec4 = [str(total)[i] for i in range(len(str(total)))]
    if dec4.count('.'):
        i = dec4.index('.')
        if (len(dec4) - i) - 1 < -(puisa + puisb):
            for j in range(-(puisa + puisb) - len(dec4) + i + 1):
                dec4.append('0')  #ajoute les 0 inutiles au produit
        dec4.pop(i)  # supprime le point décimal
        dec4[i - 1] = '%s\\Huge ,' % dec4[i - 1]  # et ajoute une Huge virgule au chiffre des unités
    lg = max(len(dec4), max(len(deca), len(decb)))  # nombre de colonnes dans le tableau
    exo.append('\\item Le produit des facteurs %s et %s.\\par' % (Affichage.decimaux(nba *
             10 ** puisa), Affichage.decimaux(nbb * 10 ** puisb)))
    cor.append('\\item Le produit des facteurs %s et %s.\\par' % (Affichage.decimaux(nba *
             10 ** puisa), Affichage.decimaux(nbb * 10 ** puisb)))
    cor.append('\\begin{enumerate}')
    cor.append(u'\\item Première méthode :\\par')
    cor.append('\\begin{tabular}[t]{*{%s}{c}}' % lg)
    cor.append('%s \\\\' % ' & '.join(ligneprod([], deca,lg)))
    cor.append('%s \\\\\n\\hline' % ' & '.join(ligneprod(['$\\times$'], decb, lg)))
    for i in range(len(dec3)):
        dec = [str(dec3[i])[j] for j in range(len(str(dec3[i])))]
        cor.append('%s \\\\' % ' & '.join(ligneprod([], dec, lg)))
    cor.append('\\hline \\\\')
    cor.append('%s \\\\' % ' & '.join(ligneprod([], dec4, lg)))
    cor.append('\\end{tabular}')
    cor.append(u'\\item Seconde méthode :\\par')
    cor.append('\\begin{tabular}[t]{*{%s}{c}}' % len(dec4))
    cor.append('%s \\\\' % ' & '.join(ligneprod([], decb, lg)))
    cor.append('%s \\\\\n\\hline' % ' & '.join(ligneprod(['$\\times$'], deca, lg)))
    for i in range(len(dec3bis)):
        dec = [str(dec3bis[i])[j] for j in range(len(str(dec3bis[i])))]
        cor.append('%s \\\\' % ' & '.join(ligneprod([], dec, lg)))
    cor.append('\\hline \\\\')
    cor.append('%s \\\\' % ' & '.join(ligneprod([], dec4, lg)))
    cor.append('\\end{tabular}')
    cor.append('\\end{enumerate}')

    #outils.Arithmetique.ecrit_tex(f1, '%s\\times%s = %s' % (Affichage.decimaux(nba *
                     #10 ** puisa, 1), Affichage.decimaux(nbb * 10 **
                     #puisb, 1), Affichage.decimaux((nba * nbb) * 10 ** (puisa +
                     #puisb), 1)), cadre=1, thenocalcul='', tabs=3)

    #### Remplacement de la fonction Arithmetique.ecrit_tex :

    formule = '%s\\times%s = %s' % (Affichage.decimaux(nba *
                     10 ** puisa, 1), Affichage.decimaux(nbb * 10 **
                     puisb, 1), Affichage.decimaux((nba * nbb) * 10 ** (puisa +
                     puisb), 1))
    cor.append((u'\\[ \\boxed{%s} \\] ').expandtabs(2 * 3) % (formule))


def Operations():
    nb_exos = 3
    tex_exos = (tex_somme, tex_difference, tex_produit)

    ordre_exos = [i for i in range(nb_exos)]

    exo = ["\\exercice", u'Poser et effectuer les opérations suivantes.', '\\begin{multicols}{2}\\noindent', '\\begin{enumerate}']
    cor = ["\\exercice*", u'Poser et effectuer les opérations suivantes.', '\\begin{multicols}{2}\\noindent', '\\begin{enumerate}']

    for i in range(nb_exos):
        a = random.randrange(nb_exos - i)
        j = ordre_exos.pop(a)
        tex_exos[j](exo, cor)
    exo.append('\\end{enumerate}')
    exo.append('\\end{multicols}')
    cor.append('\\end{enumerate}')
    cor.append('\\end{multicols}')
    return (exo, cor)

#===============================================================================
# Calcul mental
#===============================================================================


def tex_calcul_mental(exo, cor):
    modules = (plus, moins, plus, div)
    calculs = [i for i in range(20)]
    for i in range(20):
        j = random.randrange(0, len(calculs))
        (a, b) = modules[calculs[j] // 5](10)
        if calculs[j] // 5 == 0:
            choix_trou(a, b, a + b, '+', exo, cor)
        if calculs[j] // 5 == 1:
            choix_trou(a, b, a - b, '-', exo, cor)
        if calculs[j] // 5 == 2:
            choix_trou(a, b, a * b, '\\times', exo, cor)
        if calculs[j] // 5 == 3:
            choix_trou(a, b, a // b, '\\div', exo, cor)
        calculs.pop(j)


def choix_trou(nb1, nb2, tot, operateur, exo, cor):
    nbaleatoire = random.randrange(4)
    if nbaleatoire > 1:
        exo.append('\\item $%s %s %s = \\ldots\\ldots$' % (nb1,
                 operateur, nb2))
        cor.append('\\item $%s %s %s = \\mathbf{%s}$' % (nb1,
                 operateur, nb2, tot))
    elif nbaleatoire > 0:
        exo.append('\\item $%s %s \\ldots\\ldots = %s$' % (nb1,
                 operateur, tot))
        cor.append('\\item $%s %s \\mathbf{%s} = %s$' % (nb1,
                 operateur, nb2, tot))
    else:
        exo.append('\\item $\\ldots\\ldots %s %s = %s$' % (operateur,
                 nb2, tot))
        cor.append('\\item $\\mathbf{%s} %s %s = %s$' % (nb1,
                 operateur, nb2, tot))


def plus(valeurmax):
    (a, b) = (Arithmetique.valeur_alea(1, valeurmax), Arithmetique.valeur_alea(1,
              valeurmax))
    return (a, b)


def moins(valeurmax):
    (a, b) = (Arithmetique.valeur_alea(1, valeurmax), Arithmetique.valeur_alea(1,
              valeurmax))
    return (a + b, a)


def div(valeurmax):
    (a, b) = (Arithmetique.valeur_alea(1, valeurmax), Arithmetique.valeur_alea(1,
              valeurmax))
    return (a * b, a)




def CalculMental():
    exo = ["\\exercice", 'Effectuer sans calculatrice :', '\\begin{multicols}{4}\\noindent', '\\begin{enumerate}']
    cor = ["\\exercice*", 'Effectuer sans calculatrice :', '\\begin{multicols}{4}\\noindent', '\\begin{enumerate}']

    tex_calcul_mental(exo, cor)

    exo.append('\\end{enumerate}')
    exo.append('\\end{multicols}')
    cor.append('\\end{enumerate}')
    cor.append('\\end{multicols}')
    return (exo, cor)


#===============================================================================
# PRODUITS ET QUOTIENTS PAR 10, 100, 1000
#===============================================================================


def tex_dix(exo, cor):
    nb = 4  # nb de calculs de chaque type
    l = valeurs10(nb)
    for i in range(len(l)):
        j = random.randrange(0, len(l))
        tex_formule_dix(l.pop(j), exo, cor)


def tex_formule_dix(l, exo, cor):
    if l[2] == '*':
        alea = random.randrange(0, 5)
        if alea > 1:
            exo.append('\\item $%s \\quad\\times\\quad %s \\quad = \\quad \\dotfill$' %
                     (Affichage.decimaux(l[0], 1), Affichage.decimaux(l[1],
                     1)))
            cor.append('\\item $%s \\times %s = \\mathbf{%s}$' %
                     (Affichage.decimaux(l[0], 1), Affichage.decimaux(l[1],
                     1), Affichage.decimaux(l[0] * l[1], 1)))
        elif alea > 0:
            exo.append('\\item $%s \\quad\\times\\quad \\dotfill \\quad = \\quad %s$' %
                     (Affichage.decimaux(l[0], 1), Affichage.decimaux(l[0] *
                     l[1], 1)))
            cor.append('\\item $%s \\times \\mathbf{%s} = %s$' %
                     (Affichage.decimaux(l[0], 1), Affichage.decimaux(l[1],
                     1), Affichage.decimaux(l[0] * l[1], 1)))
        else:
            exo.append('\\item $\\dotfill \\quad\\times\\quad %s \\quad = \\quad %s$' %
                     (Affichage.decimaux(l[1], 1), Affichage.decimaux(l[0] *
                     l[1], 1)))
            cor.append('\\item $\\mathbf{%s} \\times %s = %s$' %
                     (Affichage.decimaux(l[0], 1), Affichage.decimaux(l[1],
                     1), Affichage.decimaux(l[0] * l[1], 1)))
    else:
        alea = random.randrange(0, 5)
        if alea > 1:
            exo.append('\\item $%s \\quad\\div\\quad %s \\quad = \\quad \\dotfill$' %
                     (Affichage.decimaux(l[0], 1), Affichage.decimaux(l[1],
                     1)))
            cor.append('\\item $%s \\div %s = \\mathbf{%s}$' % (Affichage.decimaux(l[0],
                     1), Affichage.decimaux(l[1], 1), Affichage.decimaux(l[0] /
                     l[1], 1)))
        elif alea > 0:
            exo.append('\\item $%s \\quad\\div\\quad \\dotfill \\quad = \\quad %s$' %
                     (Affichage.decimaux(l[0], 1), Affichage.decimaux(l[0] /
                     l[1], 1)))
            cor.append('\\item $%s \\div \\mathbf{%s} = %s$' % (Affichage.decimaux(l[0],
                     1), Affichage.decimaux(l[1], 1), Affichage.decimaux(l[0] /
                     l[1], 1)))
        else:
            exo.append('\\item $\\dotfill \\quad\\div\\quad %s \\quad = \\quad %s$' %
                     (Affichage.decimaux(l[1], 1), Affichage.decimaux(l[0] /
                     l[1], 1)))
            cor.append('\\item $\\mathbf{%s} \\div %s = %s$' % (Affichage.decimaux(l[0],
                     1), Affichage.decimaux(l[1], 1), Affichage.decimaux(l[0] /
                     l[1], 1)))


def valeurs10(nb):  # renvoie nb valeur de chaque type : *10, /10, *0.1
    l = []
    for i in range(nb):
        if random.randrange(0, 1):
            l.append((Arithmetique.valeur_alea(111, 999) * 10 ** random.randrange(-3,
                     0), 10 ** (i + 1), '*'))
        else:
            l.append((10 ** (i + 1), Arithmetique.valeur_alea(111, 999) * 10 **
                     random.randrange(-3, 0), '*'))
    for i in range(nb):
        l.append((Arithmetique.valeur_alea(111, 999) * 10 ** random.randrange(-3,
                 0), 10 ** (i + 1), '/'))
    for i in range(nb):
        if random.randrange(0, 1):
            l.append((Arithmetique.valeur_alea(111, 999) * 10 ** random.randrange(-3,
                     0), 10 ** (-i - 1), '*'))
        else:
            l.append((10 ** (-i - 1), Arithmetique.valeur_alea(111, 999) * 10 **
                     random.randrange(-3, 0), '*'))
    return l



def ProduitPuissanceDix():
    exo = ["\\exercice", u'Compléter sans calculatrice :', '\\begin{multicols}{2}\\noindent', '\\begin{enumerate}']
    cor = ["\\exercice*", u'Compléter sans calculatrice :', '\\begin{multicols}{2}\\noindent', '\\begin{enumerate}']

    tex_dix(exo, cor)

    exo.append('\\end{enumerate}')
    exo.append('\\end{multicols}')
    cor.append('\\end{enumerate}')
    cor.append('\\end{multicols}')
    return (exo, cor)



# -*- coding: utf-8 -*-

import random
from ..outils.Arithmetique import factorise, carrerise
from math import sqrt
from ..outils.Affichage import suppr0, suppr0list

class Poly2:
    """Classe pour les polynômes du second degré."""
    def __init__(self, a, b, c):
        if a == 0:
            print "Erreur de définition ! a doit être différent de 0."
        self.a = a
        self.b = b
        self.c = c
        self.delta = suppr0(b**2 - 4*a*c)
        if self.delta > 0:
            self.signedelta = "strictement positif"
            self.nbrac = 2
        elif self.delta < 0:
            self.signedelta = "strictement négatif"
            self.nbrac = 0
        else:
            self.signedelta = "nul"
            self.nbrac = 1

    def __add__(self, other):
        return Poly2(self.a + other.a, self.b + other.b, self.c + other.c)

    def __radd__(self, other):
        return Poly2(self.a + other.a, self.b + other.b, self.c + other.c)

    def __sub__(self, other):
        return Poly2(self.a - other.a, self.b - other.b, self.c - other.c)

    def __rsub__(self, other):
        return Poly2(self.a - other.a, self.b - other.b, self.c - other.c)

    def __str__(self):
        if self.b<=0:
            sep1 = ''
        else:
            sep1 = '+'
        if self.c<=0:
            sep2 = ''
        else:
            sep2 = '+'
        if self.a == 1:
            a1 = ''
        else:
            a1 = str(suppr0(self.a))
        if self.b == 1:
            b1 = ''
        else:
            b1 = str(suppr0(self.b))
        if self.b == 0:
            deg1 = ''
            b1 = ''
        else:
            deg1 = 'x'
        return a1 + 'x^2' + sep1 + b1 + deg1 + sep2 + str(suppr0(self.c))

    def print_signe(self, signe):
        return str(self) + " " + signe + " 0"

